/***************************************************************
 * FileName:
 *	  SelectMAP.c
 *
 * Project:
 *	  SelectMAP Configuration via embedded microprocessor.
 *
 * Copyright info:
 *
 *	  This is free software; you can redistribute it and/or modify
 *	  it as you like.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Description:
 *
 *      This is the main source file that will allow a microprocessor
 *      to configure Xilinx Virtex, Virtex-E, Virtex-EM, Virtex-II,
 *      and Spartan-II devices via the SelectMAP Configuration Mode.
 *      This code is discussed in Xilinx Application Note, XAPP502.
 *
 * History:
 *	  3-October-2001  MN/MP  - Created
 ****************************************************************/



 //**********************************************************************
 //  Virtex Configuration Constants (Global Variables)
 //**********************************************************************


 //CPLD Register Addresses:

 WordPtr CPLD_Config_Reg 		= 0x29000000;
 WordPtr CPLD_Program_Reg  		= 0x29000002;
 WordPtr CPLD_Input_Reg 		= 0x29000004;

 //Constants:

 //Mask Values for CCLK bit in the Configuration Register:
 //Used in SelectMAP configuration

 Word CCLK_LOW_MASKVALUE  = 0xFFFE;
 Word CCLK_HIGH_MASKVALUE = 0x0001;


 //Program Register:

 int CS_Bit		 	= 0x0004;
 int Write_Bit		= 0x0002;
 int Prog_Bit		= 0x0001;





  /***********************************************************************
   *
   * Function:     Start_SelectMAP
   *
   * Description:  Begins SelectMAP configuration of Xilinx FPGA
   *
   *
   * Parameters:
   *	  void
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  Main()
   *
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/



  void
  Start_SelectMAP( void )
  {

  	//Address Range varaibles
  	WordPtr flash_start_addr;
  	WordPtr flash_end_addr;

  	//Location of configuration data stored in flash
  	flash_start_addr = 0x28000200;
  	flash_end_addr   = 0x280156FA;

 	//----------------------------------------------------------------------
 	// SelectMAP()
 	//----------------------------------------------------------------------

  	SelectMAP(flash_start_addr, flash_end_addr);

  	//----------------------------------------------------------------------
  	// Monitor the DONE bit in the CPLD Input Register to see if
  	// configuration successful
  	//----------------------------------------------------------------------

  	Check_DONE_Bit();


  }


 /***********************************************************************
  *
  * Function:     SelectMAP
  *
  * Description:  Initiates SelectMAP Configuration.
  *               Calls SelectMAP_Output() to output MSByte and LSByte
  *
  * Parameters:
  *	  flash_start_addr = Address value where Virtex bitstream begins
  *   flash_end_addr   = Address value where Virtex bitstream ends
  *
  * Returns:
  *	  void
  *
  * Called By:
  *	  Start_SelectMAP
  *
  * History:
  *	  3-October-2001  MN/MP  - Created
  *
  ***********************************************************************/


 void
 SelectMAP( WordPtr flash_start_addr, WordPtr flash_end_addr )
 {
 	Word Data16;
 	WordPtr flash_addr;


 	//----------------------------------------------------------------------
 	//Toggle Program_OE, /CS, and /WRITE  (Bring all three bits LOW)
 	//This is accomplished by writing to the Program Register in the CPLD.
 	//Note that /CS and /WRITE bits in the Program Register have been
 	//configured to preset upon powerup.
	//
 	//NOTE: The Program_OE bit should be driven high to bring the
 	//      Virtex Program Pin low.  Likewise, it should be driven
 	//      low to bring the Virtex Program Pin to High-Z
 	//----------------------------------------------------------------------


	//Bring CS, WRITE and PROGRAM_OE LOW:
 	IOWrite( (WordPtr) CPLD_Program_Reg, Prog_Bit );



 	//----------------------------------------------------------------------
 	//Bring Program High
 	//But leave /CS and /WRITE Low:
 	//----------------------------------------------------------------------

	//Program_OE bit Low brings the Virtex Program Pin to High Z:
 	IOWrite(CPLD_Program_Reg, 0x0000);



 	//----------------------------------------------------------------------
 	//Check for /INIT after Program is High Z:
 	//----------------------------------------------------------------------


 	Data16 = 0;  //Initialize Data16 variable before entering While Loop

 	while(Data16 == 0){
 		Data16 = IORead(CPLD_Input_Reg);  //Read Input Register
 		Data16 = Data16 & 0x0001;    	  //Check Status of /INIT
 		}


 	//----------------------------------------------------------------------
 	//Begin SelectMAP Configuration
 	//----------------------------------------------------------------------


 	for(flash_addr = flash_start_addr; flash_addr <= flash_end_addr; flash_addr++){
 		Data16 = IORead(flash_addr); //Read Flash

 		//Pass the 16-bit Word to SelectMAP_output() function:
 		SelectMAP_output(Data16);
 		}
 }





  /***********************************************************************
   *
   * Function:     SelectMAP_output
   *
   * Description:  This function takes a byte-swapped
   *  			   16-bit configuration word, and outputs the MSByte and
   *               LSByte in a Byte-wide, serial fashion.  This function is
   *               also responsible for strobing the
   *               Configuration Clock (CCLK)
   *
   * Parameters:
   *	  Data16 = 16 bit configuration word previously stored in flash
   *      		   memory
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  SelectMAP()
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/

 void
 SelectMAP_output( Data16 )
 {
 	Word temp;
 	Word DataOut;
 	int Busy_Check;
 	int busy_count;



 	//----------------------------------------------------------------------
 	//MS Byte:
 	//----------------------------------------------------------------------

 	//Write to the Configuration Register:
 		//	1)  The value of MS Byte
 		//	2)  CCLK = Low


 	IOWrite(CPLD_Config_Reg, Data16 & CCLK_LOW_MASKVALUE);


 	//Keep MSByte on Databus, but Bring CCLK High:


 	IOWrite(CPLD_Config_Reg, Data16 | CCLK_HIGH_MASKVALUE);


 	//----------------------------------------------------------------------
 	//  Optional:  Check for BUSY
 	//----------------------------------------------------------------------

 	Check_BUSY(Data16);  //Check BUSY pin for this Upper Byte


 	//----------------------------------------------------------------------
 	//LS Byte:
 	//----------------------------------------------------------------------


 	//Shift Left  (i.e.  '0xABCD << 8' will become 0xCD00)

 	Data16=Data16 << 8;

 	//Write to the Configuration Register:
 		//	1)  The value of LS Byte
 		//  2)  CCLK = Low


 	IOWrite(CPLD_Config_Reg, Data16 & CCLK_LOW_MASKVALUE);


 	//Bring CCLK High:


 	IOWrite(CPLD_Config_Reg, Data16 | CCLK_HIGH_MASKVALUE);

 	//----------------------------------------------------------------------
  	//  Optional:  Check for BUSY
  	//----------------------------------------------------------------------

 	Check_BUSY(Data16);  //Check BUSY pin for this Lower Byte

 }



  /***********************************************************************
   *
   * Function:     Check_Busy  (OPTIONAL)
   *
   * Description:  This function takes monitors the CPLD Input Register
   *  			   by checking the status of the BUSY bit in that Register.
   *               By doing so, it monitors the Xilinx Virtex device's
   *               BUSY Pin. This function may not be necessary depending
   *               upon the speed of the microprocessor.
   *
   *               The maximum speed of SelectMAP configuration
   *               is different for different FPGA families.
   *               Please refer to the appropriate data sheet.
   *
   * Parameters:
   *	  void
   *
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  SelectMAP_output()
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/


void
Check_BUSY(Word Data16) {

	Word Busy_Check;

			//----------------------------------------------------------------------
			//Check For BUSY:
			//	If BUSY is HIGH, keep giving CCLK's.
			//  Else, return and send the next Byte
			//----------------------------------------------------------------------

			do{
				Busy_Check = IORead(CPLD_Input_Reg);  //Read Input Register
				Busy_Check = Busy_Check >> 2;

				if(Busy_Check==0x0001){    //If Busy is High,

					//Bring CCLK Low:
					IOWrite(CPLD_Config_Reg, Data16 & CCLK_LOW_MASKVALUE);
					//Bring CCLK HIgh:
					IOWrite(CPLD_Config_Reg, Data16 | CCLK_HIGH_MASKVALUE);
					}

				}while(Busy_Check == 0x0001); //Keep doing this until Busy is low
}





  /***********************************************************************
   *
   * Function:     Check_Done_Bit
   *
   * Description:  This function takes monitors the CPLD Input Register
   *  			   by checking the status of the DONE bit in that Register.
   *               By doing so, it monitors the Xilinx Virtex device's DONE
   *               Pin to see if configuration bitstream has been properly
   *               loaded
   *
   * Parameters:
   *	  void
   *
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  Start_SelectMAP()
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/



void Check_DONE_Bit(void){

	Word Data16;

	//Read CPLD Input Register:
	Data16 = IORead(CPLD_Input_Reg);

	//Check the DONE bit of the CPLD Input Register:
	Data16 = (Data16 & 0x0002) ? 1 : 0;

	if (Data16 == 1){					    //If DONE Pin is High, display message
			FrmAlert (resAlertIDSuccess);   //"Successful! DONE is High!"
		}
	else{	//Otherwise, if DONE is low, device is not configured
			Data16 = IORead(CPLD_Input_Reg);
			//Check the INIT bit of the CPLD Input Register
			Data16 = (Data16 & 0x0001) ? 1 : 0;

			if(Data16 == 1) { 					  //If INIT is High,
				FrmAlert (resAlertIDFailed_and_InitHigh);	//"Configuration Failed", DONE is Low, Init is High"
			}
			else{
				FrmAlert (resAlertIDFailed_and_InitLow);	//"Configuration Failed", DONE is Low, Init is Low"
			}
		}
}



