/***************************************************************
 * FileName:
 *	  SlaveSerial.c
 *
 * Project:
 *	  SlaveSerial Configuration via embedded microprocessor.
 *
 * Copyright info:
 *
 *	  This is free software; you can redistribute it and/or modify
 *	  it as you like.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Description:
 *
 *      This is the main source file that will allow a microprocessor
 *      to configure Xilinx Virtex, Virtex-E, Virtex-EM, Virtex-II,
 *      and Spartan-II devices via the SlaveSerial Configuration Mode.
 *      This code is discussed in Xilinx Application Note, XAPP502.
 *
 * History:
 *	  3-October-2001  MN/MP  - Created
 ****************************************************************/





 //**********************************************************************
 //  Virtex Configuration Constants (Global Variables)
 //**********************************************************************


 //CPLD Register Addresses:

 WordPtr CPLD_Config_Reg 		= 0x29000000;
 WordPtr CPLD_Program_Reg  		= 0x29000002;
 WordPtr CPLD_Input_Reg 		= 0x29000004;

 //Constants:

 //Mask Values for CCLK bit in the Configuration Register:
 //Used in SelectMAP configuration

 Word CCLK_LOW_MASKVALUE  = 0xFFFE;
 Word CCLK_HIGH_MASKVALUE = 0x0001;


 //Program Register:

 int CS_Bit		= 0x0004;
 int Write_Bit		= 0x0002;
 int Prog_Bit		= 0x0001;





  /***********************************************************************
   *
   * Function:     Start_SlaveSerial
   *
   * Description:  Begins Slave Serial configuration of Xilinx FPGA
   *
   *
   * Parameters:
   *	  void
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  Main()
   *
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/



void
Start_SlaveSerial( )
{

  	//Address Range varaibles
  	WordPtr flash_start_addr;
  	WordPtr flash_end_addr;

  	//Location of configuration data stored in flash
	flash_start_addr = 0x28000200;
	flash_end_addr   = 0x280156FA;

 	//----------------------------------------------------------------------
 	// SlaveSerial()
 	//----------------------------------------------------------------------

	SlaveSerial(flash_start_addr, flash_end_addr);


  	//----------------------------------------------------------------------
  	// Monitor the DONE bit in the CPLD Input Register to see if
  	// configuration successful
  	//----------------------------------------------------------------------

	Check_DONE_Bit();

}






 /***********************************************************************
  *
  * Function:     SlaveSerial
  *
  * Description:  Initiates SlaveSerial Configuration.
  *               Calls ShiftDataOut() to output serial data
  *
  * Parameters:
  *	  flash_start_addr = Address value where Virtex bitstream begins
  *   flash_end_addr   = Address value where Virtex bitstream ends
  *
  * Returns:
  *	  void
  *
  * Called By:
  *	  Start_SlaveSerial
  *
  * History:
  *	  3-October-2001  MN/MP  - Created
  *
  ***********************************************************************/




void
SlaveSerial( WordPtr flash_start_addr, WordPtr flash_end_addr )
{
	Word Data16;
	WordPtr flash_addr;


	//----------------------------------------------------------------------
	//Toggle Program Pin by Toggling Program_OE bit
	//This is accomplished by writing to the Program Register in the CPLD
	//
	//NOTE: The Program_OE bit should be driven high to bring the Virtex
	//      Program Pin low. Likewise, it should be driven low
	//      to bring the Virtex Program Pin to High-Z
	//----------------------------------------------------------------------

	IOWrite(CPLD_Program_Reg, Prog_Bit); //PROGRAM_OE LOW



	//----------------------------------------------------------------------
	//Bring Program High-Z
	//(Drive Program_OE bit low to bring Virtex Program Pin to High-Z
	//----------------------------------------------------------------------

	//Program_OE bit Low brings the Virtex Program Pin to High Z:
	IOWrite(CPLD_Program_Reg, 0x0000);



	//----------------------------------------------------------------------
	//Check for /INIT after Program is High Z:
	//----------------------------------------------------------------------


	Data16 = 0;  //Initialize Data16 variable before entering While Loop

	while(Data16 == 0){
		Data16 = IORead(CPLD_Input_Reg);    //Read Input Register
		Data16 = Data16 & 0x0001;		    //Check Status of /INIT
		}


	//----------------------------------------------------------------------
	//Begin Slave-Serial Configuration
	//----------------------------------------------------------------------


	for(flash_addr = flash_start_addr; flash_addr <= flash_end_addr; flash_addr++){
		Data16 = IORead(flash_addr); //Read Flash
		ShiftDataOut(Data16);		 //Pass 16-bit data to ShiftDataOut function
		}
}




  /***********************************************************************
   *
   * Function:     ShiftDataOut
   *
   * Description:  This function takes a 16-bit configuration word,
   *               and serializes it, MSB first, LSB Last
   *               This function is also responsible for strobing the
   *               Configuration Clock (CCLK)
   *
   * Parameters:
   *	  Data16 = 16 bit configuration word previously stored in flash
   *      		   memory
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  Slave_Serial()
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/

void
ShiftDataOut( Data16 )
{

	Word DataOut;
	Word SCLK_LOW;
	Word SCLK_HIGH;

	SCLK_LOW  = 0x0000;
	SCLK_HIGH = 0x0002;

	//Upper Byte:

	DataOut = (Data16 & 0x0100) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //1st Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0200) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //2nd Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0400) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //3rd Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0800) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //4th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x1000) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //5th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x2000) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //6th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x4000) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //7th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x8000) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //8th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

// --  Lower Byte:

	DataOut = (Data16 & 0x0001) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //9th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0002) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //10th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0004) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //11th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0008) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //12th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0010) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //13th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0020) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //14th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0040) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //15th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

	DataOut = (Data16 & 0x0080) ? 1 : 0;
	IOWrite(CPLD_Config_Reg, SCLK_LOW  | DataOut );  //16th Bit, and Bring SCLK Low
	IOWrite(CPLD_Config_Reg, SCLK_HIGH | DataOut );  //SCLK High

}




BE CAREFUL THIS IS BYTESWAPPED!



  /***********************************************************************
   *
   * Function:     Check_Done_Bit
   *
   * Description:  This function takes monitors the CPLD Input Register
   *  			   by checking the status of the DONE bit in that Register.
   *               By doing so, it monitors the Xilinx Virtex device's DONE
   *               Pin to see if configuration bitstream has been properly
   *               loaded
   *
   * Parameters:
   *	  void
   *
   *
   * Returns:
   *	  void
   *
   * Called By:
   *	  Start_SlaveSerial()
   *
   * History:
   *	  3-October-2001  MN/MP  - Created
   *
   ***********************************************************************/



void Check_DONE_Bit(void){

	Word Data16;

	//Read CPLD Input Register:
	Data16 = IORead(CPLD_Input_Reg);

	//Check the DONE bit of the CPLD Input Register:
	Data16 = (Data16 & 0x0002) ? 1 : 0;

	if (Data16 == 1){					    //If DONE Pin is High, display message
			FrmAlert (resAlertIDSuccess);   //"Successful! DONE is High!"
		}
	else{	//Otherwise, if DONE is low, device is not configured
			Data16 = IORead(CPLD_Input_Reg);
			//Check the INIT bit of the CPLD Input Register
			Data16 = (Data16 & 0x0001) ? 1 : 0;

			if(Data16 == 1) { 					  //If INIT is High,
				FrmAlert (resAlertIDFailed_and_InitHigh);	//"Configuration Failed", DONE is Low, Init is High"
			}
			else{
				FrmAlert (resAlertIDFailed_and_InitLow);	//"Configuration Failed", DONE is Low, Init is Low"
			}
		}
}



